@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2006 Stas Zykiewicz <stas.zytkiewicz@gmail.com>
+# Copyright (c) 2006 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
 #
 #           utils.py
 # This program is free software; you can redistribute it and/or
@@ -20,30 +20,48 @@
 import __builtin__
 import imp
 import sys
-
+from SPConstants import *
+from NormalDistTable import normal_distribution_table
 import gettext
 import glob
+import fnmatch
 import locale
 import logging
 import os
-import pangofont
+if not NoGtk:
+    import pangofont
 import pygame
 import shutil
 import time
 import datetime
 import types
-from SPConstants import *
+import ConfigParser
+import math
+from collections import MutableMapping
 from pygame.constants import *
+from pygame import surfarray
+import textwrap
+#from facedetect.facedetect import CAMFOUND
+CAMFOUND = None
+if CAMFOUND:
+    import opencv.adaptors as adaptors
+module_logger = logging.getLogger("childsplay.utils")
+
+if NoGtk:
+    module_logger.warning("NoGtk is set, not using GTK pango font rendering some non-latin characters might be missing.")
 
-module_logger = logging.getLogger("schoolsplay.utils")
 
 UT_DEBUG = 0
 
+class StopGameException(Exception):
+    # Raise this to quit outside the eventloop
+    pass
+
 class MyError(Exception):
-    def __init__(self, str, line=''):
+    def __init__(self, str='', line=''):
         self.line = line
         self.str = str
-    def __str__(self): return "%s: %s" % (self.line, self.str)
+    def __str__(self): return "%s" % self.str
 
 class SPError(Exception):
     pass
@@ -53,7 +71,34 @@ class ReporterError(SPError):
     pass
 class XMLParseError(SPError):
     pass
-    
+class StopmeException(Exception):
+    pass
+class RestartMeException(Exception):
+    pass
+class TextRectException(Exception):
+    def __init__(self, message = None):
+        self.message = message
+    def __str__(self):
+        return self.message
+
+###### For debugging memoryleaks #####################
+#def memory_profile(h):
+#    f = open('heap_profile.txt', 'a')
+#    f.write('heap: \n%s' % h.heap())
+#    f.write('\n\n')
+#    hh = h.heap()
+#    #f.write('heap byvia: \n%s' % hh.byvia)
+#    f.write('Traversing Heap: \n%s' % hh.get_rp())
+#    f.write('\n\n')
+#    f.close()
+#####################################################
+
+def calcdist(v1, v2):
+    """Calculate the distance between two vectors where a vector is a tuple (x,y)"""
+    dx = v2[0] - v1[0]
+    dy = v2[1] - v1[1]
+    return math.sqrt(dx*dx + dy*dy)
+
 def set_locale(lang=None):
     """Used by the core to set the locale.
     """
@@ -69,15 +114,16 @@ def set_locale(lang=None):
                 if os.environ.has_key('LANGUAGE'):
                     lang = os.environ['LANGUAGE'].split(':')[0]
                 else:
-                    lang = locale.getdefaultlocale()[0]
+                    lang, enc = locale.getdefaultlocale()
+                    lang = "%s.%s" % (lang, enc.lower())
             except ValueError, info:
                 module_logger.error(info)
                 lang = 'en'
         languages = [lang]
         if os.environ.has_key('LANGUAGE'):
             languages += os.environ['LANGUAGE'].split(':')
-        module_logger.info("Setting childsplay locale to '%s' modir: %s" % (lang, LOCALEDIR))
-        lang_trans = gettext.translation('childsplay_sp', \
+        module_logger.info("Setting seniorplay locale to '%s' modir: %s" % (lang, LOCALEDIR))
+        lang_trans = gettext.translation('seniorplay', \
             localedir=LOCALEDIR, \
             languages=languages)
         __builtin__.__dict__['_'] = lang_trans.ugettext
@@ -87,7 +133,7 @@ def set_locale(lang=None):
             txt = "Cannot set language to '%s' \n switching to English" % lang
             module_logger.info("%s %s" % (info, txt))
         __builtin__.__dict__['_'] = lambda x:x
-        lang = 'en_US'
+        lang = 'en_US.utf8'
     else:
         lang = lang.split('@')[0]#.split('.')[0].split('_')[0]
     # This is to signal that we running under a RTL locale like Hebrew or Arabic
@@ -97,7 +143,12 @@ def set_locale(lang=None):
     LANG = lang
     module_logger.debug("Locale set to %s, RTL set to %s" % (LANG, LOCALE_RTL))
     return (lang, LOCALE_RTL)
-    
+
+def get_locale_local():
+    """Returns the locale set by set_locale.
+    This can differ from systems locale."""
+    return (LANG, LOCALE_RTL)
+
 def get_locale():
     """Get the systems locale.
     This can be different from the locale used by childsplay.(see set_locale())"""
@@ -124,7 +175,26 @@ def get_locale():
     elif lang in ['he', 'ar']:
         LOCALE_RTL = True
     return (lang, LOCALE_RTL)
-    
+
+def read_rcfile(path):
+    """Reads a config file and returns all the entries in a hash.
+    Use this for the activity rc files.
+    The core libs have their own specialized rc readers.
+    """
+    config = ConfigParser.ConfigParser()
+    if not os.path.exists(path):
+        module_logger.info("No rc file found at: %s" % path)
+        return {}
+    try:
+        config.read(path)
+    except Exception, info:
+        module_logger.error("Failed to parse rc file: %s" % info)
+        return {}
+    hash = {}
+    for section in config.sections():
+        hash[section] = dict(config.items(section))
+    return hash
+
 def read_unicode_file(path, lines=1):
     import codecs
     file = []
@@ -180,17 +250,6 @@ def map_keys(key_map, key):
             module_logger.error("No key: %s found" % key)
             return key
 
-def replace_rcfile():
-    """This is only used when we want to replace an existing configfile."""
-    #pass #enable this and comment out the rest when not replacing
-    src = os.path.join(RCDIR, CHILDSPLAYRC)
-    dst = os.path.join(HOMEDIR, 'ConfigData', CHILDSPLAYRC)
-    if os.path.exists(src) and os.path.exists(dst):
-        shutil.move(dst, dst + '.old')
-        module_logger.info("Backup made from your old config file called %s." % dst + '.old')
-        module_logger.info("Replace childsplayrc file:\n %s\n->%s" % (src, dst))
-        shutil.copyfile(src, dst)
-
 class NoneSound:
     """Used by the load_sound and load_music functions to provide
      a bogus sound object.
@@ -204,7 +263,7 @@ class NoneSound:
         pass
     def get_volume(self):
         return None
-    def set_volume(sefl, sound_value):
+    def set_volume(self, sound_value):
         pass
     def __eq__(self, other):
         if other == 'NoneSound': return True
@@ -309,8 +368,44 @@ def load_music(file):
         playmusic = MusicObject(file)
 
     return playmusic
+    
+def aspect_scale(img,(bx,by)):
+    """ Scales 'img' to fit into box bx/by.
+     This method will retain the original image's aspect ratio """
+    ix,iy = img.get_size()
+    if ix > iy:
+        # fit to width
+        scale_factor = bx/float(ix)
+        sy = scale_factor * iy
+        if sy > by:
+            scale_factor = by/float(iy)
+            sx = scale_factor * ix
+            sy = by
+        else:
+            sx = bx
+    else:
+        # fit to height
+        scale_factor = by/float(iy)
+        sx = scale_factor * ix
+        if sx > bx:
+            scale_factor = bx/float(ix)
+            sx = bx
+            sy = scale_factor * iy
+        else:
+            sy = by
+    return pygame.transform.scale(img, (int(sx),int(sy)))
 
-def load_image(file, transparent=0, alpha=0, theme='default'):
+def grayscale_image(surf):
+    width, height = surf.get_size()
+    for x in range(width):
+        for y in range(height):
+            red, green, blue, alpha = surf.get_at((x, y))
+            L = 0.3 * red + 0.59 * green + 0.11 * blue
+            gs_color = (L, L, L, alpha)
+            surf.set_at((x, y), gs_color)
+    return surf
+    
+def load_image(file, transparent=0, theme='childsplay'):
     """loads an image and convert it.
     When a theme is set it will try to load the image from the theme directory
     otherwise it will load the default image.
@@ -319,42 +414,47 @@ def load_image(file, transparent=0, alpha=0, theme='default'):
     If you have a image with an alpha channel set transparent to 0 nd alpha to 1.
     ATTENTION, if you planning to rotate your image DON'T use alpha, it will crash pygame."""
     
-    if theme and theme != 'default':
+    if theme and theme != 'childsplay':
         path, name = os.path.split(file)
         path = os.path.join(path, theme, name)
         try:
             surface = pygame.image.load(path)
         except pygame.error:
-            module_logger.error('Failed to load theme image: %s' % path)
-            module_logger.error('%s' % pygame.get_error())
+            module_logger.info('Failed to load theme image: %s' % path)
+            module_logger.info('%s' % pygame.get_error())
+            module_logger.info('Trying default images')
             theme = None
-    if not theme or theme == 'default':
+    if not theme or theme == 'childsplay':
         try:
             surface = pygame.image.load(file)
         except pygame.error:
-            module_logger.error('Could not load image "%s"\n %s' % (file, pygame.get_error()))
+            module_logger.exception('Could not load image "%s"\n %s' % (file, pygame.get_error()))
             MyError.line = 'Utils.py -> load_image()'
             raise MyError('Could not load image "%s"\n %s' % (file, pygame.get_error()))
     if transparent:
         surface.set_colorkey(surface.get_at((0, 0)), RLEACCEL)
-    if alpha:
+    if surface.get_alpha():
         #if the image loaded has alpha, the overall alpha set here
         #(128) will be overriden. Do it anyway for forcing RLEACCEL
-        surface.set_alpha(128, RLEACCEL)
+        surface.set_alpha(127, RLEACCEL)
         # if the display is in a lower mode, then surface.convert would
         # convert the surface in an alpha-less surface.
         return surface
     return surface.convert()
 
-def char2surf(char, fsize, fcol=None, ttf=None, bold=False, antialias=True, split=0):
+def char2surf(char, fsize, fcol=None, ttf='arial', bold=False, antialias=True, split=0):
     """Renders a character and returns the surface.
     char must be a string. Returns the surface.
     This now uses pango to render the fonts.
     @ttf must be a fontname installed on the system
+    When you supply a name ending on '.ttf' a truetype font path is assumed and
+    the pygame font renderer is used. 
     @split indicates the length of the strings. When it's set to 0, the default,
     it will not split the string and the surface will be returnt.
     When split > 0 a list with surfaces is returnt.
     """
+    if NoGtk:
+        ttf = TTF
     items = []
     if split:
         txt = txtfmt([char], split)
@@ -362,36 +462,34 @@ def char2surf(char, fsize, fcol=None, ttf=None, bold=False, antialias=True, spli
         txt = [char]
     if not fcol:
         fcol = (0, 0, 0)
-    longest = 0
     if ttf and os.path.splitext(ttf)[1] == '.ttf':
-        module_logger.debug('ttf=%s, using standard pygame font' % ttf)
-        font = pygame.font.Font(None, fsize)
+        #module_logger.debug('ttf=%s, using standard pygame font' % ttf)
+        font = pygame.font.Font(ttf, fsize)
         for line in txt:
+            if not txt: continue
             s = font.render(line, True, fcol)
             items.append(s)
-            if s.get_width() > longest:
-                longest = s.get_width()
     else:
-        font = pangofont.PangoFont(family=ttf, size=fsize)
+        font = pangofont.PangoFont(family=ttf, size=fsize, bold=bold)
         font.set_bold(bold)
         for line in txt:
             s = font.render(line, True, fcol, None)
             items.append(s)
-            if s.get_width() > longest:
-                longest = s.get_width()
     if split:
         return items
     else:
         return items[0]
     
-def font2surf(word, fsize, fcol=None, ttf=None, sizel=None, bold=False, antialias=True):
+def text2surf(word, fsize, fcol=None, ttf=None, sizel=None, bold=False, antialias=True):
     """Renders a text in a surface and returns a surface,size list of the items
     sizelist is a list like this [(7,17),(10,17)] tuples are x,y size of the character."""
+    if NoGtk:
+        ttf = TTF
     if not fcol:
         fcol = (0, 0, 0)
     if ttf and os.path.splitext(ttf)[1] == '.ttf':
         module_logger.debug('ttf=%s, using standard pygame font' % ttf)
-        font = pygame.font.Font(None, fsize)
+        font = pygame.font.Font(ttf, fsize)
         s = font.render(word, True, fcol)
     else:
         try:
@@ -413,14 +511,36 @@ def font2surf(word, fsize, fcol=None, ttf=None, sizel=None, bold=False, antialia
         sizelist = map(font.size, word)
     return s, sizelist
 
-def get_files(basedir, match):
-    """ Obsolete, use glob.glob.
-     This will return the result of 
-     glob.glob(os.path.join(basedir,match))
-     """
-    files = glob.glob(os.path.join(basedir, match))
-    return files
-    
+def Ipl2NumPy(img):
+    """ Converts an openCV cvArray to a pygame surface.
+     Parameters:   IplImage img - image"""
+    array = adaptors.Ipl2NumPy(img)
+    surf = pygame.surfarray.make_surface(array)
+    surf = pygame.transform.rotate(surf, -90)
+    surf = pygame.transform.flip(surf, 1, 0)
+    return surf
+
+def NumPy2Ipl(surf):
+    """Converts a pygame surface to an openCV cvArray.
+    Parameters:   pygame.Surface surf - pygame Surface
+    """
+    py_image = surfarray.pixels3d(surf)
+    cv_image = adaptors.NumPy2Ipl(py_image.transpose(1,0,2))
+    return cv_image
+
+def find_files(directory, patternlist):
+    """Generator function to search files that match a certain pattern.
+    patternlist must be a list of patterns.
+    Usage: for file in find_files('topdir','*.foo'):
+                print 'found:',file 
+    """
+    for root, dirs, files in os.walk(directory):
+        for pattern in patternlist:
+            for basename in files:
+                if fnmatch.fnmatch(basename, pattern):
+                    filename = os.path.join(root, basename)
+                    yield filename
+
 class Dimmer:
     """
 Dimmer class
@@ -573,117 +693,29 @@ def import_module(filename, globals=None, locals=None, fromlist=None):
         raise MyError, info
         if fp: fp.close()
     
-#class EntryEdit:
-#    """This class must be implemented by the childsplay Score class.
-#    Class provides a RTL aware'entry edit' object which renders the users
-#    input onto the screen. This screen is updated by this class with a call to
-#    pygame.display.update(the changed rect)
-#    The font, size and color is taken from the parent class.
-#    """
-#    
-#    def __init__(self,parent,screen,pos,prompt,loc):
-#        self.locale_rtl,self.stdloc = loc
-#        self.surf=screen
-#        self.prompt=prompt
-#        self.parent = parent
-#        self.sc_fsize = self.parent.sc_fsize
-#        self.sc_fcol = self.parent.sc_fcol
-#        self.hsc_ttf = self.parent.hsc_ttf
-#        ### RTL mapping related ######
-#        self.key_map = None # Used as the default keymapping, see function utils.map_keys()
-#        if UT_DEBUG: print "self.locale_rtl",self.locale_rtl,"stdloc",self.stdloc
-#        if self.locale_rtl and self.stdloc:
-#            self.key_map = {'he':he_key_map,
-#                        'ar':ar_key_map}[self.stdloc]
-#        if self.stdloc == 'ru': self.key_map = ru_key_map
-#        ### end ####
-#        # Prevent from loops from using 100% CPU
-#        self.clock = pygame.time.Clock()
-#        self.pos = pos
-#        self.maxlen = 16
-#        
-#        self.rect = pygame.Rect(pos[0],pos[1],700,100)
-#        
-#    def _get_key(self):
-#        while 1:
-#            self.clock.tick(30)
-#            event = pygame.event.poll()
-#            if event.type == KEYDOWN:
-#                m = pygame.key.get_mods()
-#                # when using key_maps : < > and ? need special care (keys 59,47,46,44)
-#                if self.locale_rtl and event.key == 59 or event.key == 47 or \
-#                            event.key == 46 or event.key == 44:
-#                    return event.key
-#                if m & KMOD_SHIFT:
-#                    return event.key-32
-#                return event.key
-#            else:
-#                pass
-#                
-#    def ask(self):
-#        """ask() -> answer
-#        This will renders the question and answer onto the screen and return
-#        the answer when the user hits enter.
-#        The anwser can be max 16 characters"""
-#                
-#        current_string = []
-#        self.display_box(self.prompt + string.join(current_string,""))
-#        while 1:
-#            inkey = self._get_key()
-#            if inkey == K_BACKSPACE:
-#                current_string = current_string[0:-1]
-#            elif inkey == K_RETURN:
-#                break        
-#            elif len(current_string) < self.maxlen and 31 < inkey < 127:
-#                current_string.append(map_keys(self.key_map,chr(inkey)))
-#            self.display_box(self.prompt+string.join(current_string,""))
-#        return string.join(current_string,"")
-#    
-#    def display_box(self,message):
-#        """Print a message on the screen"""
-#        pygame.draw.rect(self.surf, (0,0,0),self.rect, 0)
-#        ## For testing only
-#        #pygame.draw.rect(self.surf, (255,255,255),self.rect, 1)
-#        if len(message) != 0:
-#            s,spam = font2surf(message,self.sc_fsize,self.sc_fcol,self.hsc_ttf)
-#            if self.locale_rtl:
-#                self.surf.blit(s,(self.rect[2]-s.get_width(),self.rect[1]+4))
-#            else:
-#                self.surf.blit(s,(self.rect[0]+4,self.rect[1]+4))
-#        pygame.display.update()
-
-
-def __string_split(line, split):
-    """ Split a string at the first space left from split index
-      in two pieces of 'split' length -> two strings,
-      where the first is of length 'split'.
-      The second string is the the rest or None
-      line = string
-      split = integer
-      """
-    while line[split] != " " or split == 0:
-        split -= 1
-    line1 = line[:split]
-    line2 = line[split + 1:]#loose the space
-    
-    return line1, line2
-
 def txtfmt(text, split):
     """ Formats a list of strings in a list of strings of 'split' length.
-       returns a new list of strings. This depends on utils.__string_split().
+       returns a new list of strings.
        text = list of strings
        split = integer
        """
-    newtxt = []
-    left, right = "", ""
+    return textwrap.wrap('\n'.join(text), split)
+
+def txtfmtlines(text, split):
+    """Formats a list of strings of 'split' length.
+    Returns a list of strings with single linebreaks preserved.
+    This differs from txtfmt which removed single line linebreaks
+    """
+    wrapper = textwrap.TextWrapper()
+    wrapper.width = split
+    wrapper.drop_whitespace = False
+    lines = []
     for line in text:
-        while len(line) > split:
-            left, right = __string_split(line, split)
-            newtxt.append((left.strip(' ')))
-            line = right
-        newtxt.append((line))
-    txt = filter(None, newtxt)
-    return txt
+        if not line.split('\n'):
+            lines += line
+        else:
+            lines += wrapper.wrap(line)
+    return lines
 
 class ScaleImages:
     def __init__(self, imgObjects, TargetSize, stdCardObj=None):
@@ -696,7 +728,7 @@ class ScaleImages:
                    the scaled images. Else the images are blitted on this object.
         TargetSize = desired scale size (tuple)
         """
-        self.logger = logging.getLogger("schoolsplay.utils.ScaleImages")
+        self.logger = logging.getLogger("childsplay.utils.ScaleImages")
         self.TargetSize = pygame.Rect((0, 0) + TargetSize)# convert this tuple in a pygame rect
             # see: def _scale_if_needed
         if stdCardObj:
@@ -704,24 +736,23 @@ class ScaleImages:
             # just return scaled images.
             border = 10 # border size of card (memory), 2 * border size
             try: 
-                    self.stdCard = stdCardObj
-                    # scale the card also to TargetSize, so we use one card for everything
-                    self.scaled_card = self._scale_if_needed(self.stdCard)
-                    self.TargetSize.inflate_ip(-border, -border)# reduce the size of the rect, so the images are smaller
-                        # then the card (compensate for border)
+                self.stdCard = stdCardObj
+                # scale the card also to TargetSize, so we use one card for everything
+                self.scaled_card = self._scale_if_needed(self.stdCard)
+                self.TargetSize.inflate_ip(-border, -border)# reduce the size of the rect, so the images are smaller
+                    # then the card (compensate for border)
             except StandardError, info:
-                    self.logger.exception("Failed to scale stdCardObj:%s, %s" % (stdCardObj, info))
-                    self.stdCard = None
+                self.logger.exception("Failed to scale stdCardObj:%s, %s" % (stdCardObj, info))
+                self.stdCard = None
         else:
             self.stdCard = None #to use as a test later, if we gonna blit the img on a card or not
                 # see get_images
-         
         self.imgObjects = imgObjects
         
     def get_images(self, image=None):
         """ This returns a list with scaled images, blitted on a card if
          it was parsed to the class constructor.
-         When image is a surface image is acaled on a card and returnt in list of one
+         When image is a surface image is scaled on a card and returnt in list of one
         """
         if type(self.imgObjects) == types.DictType:
             # This won't work in combination with stdCard
@@ -735,8 +766,10 @@ class ScaleImages:
         if self.stdCard:# we have a blanc card to blit the images on
             card_imgs = []
             for img in imgs:
-                card = self.scaled_card.convert() # convert() returns a copy, not a reference
-                    # this way we keep a 'clean' card
+                if self.scaled_card.get_alpha():
+                    card = self.scaled_card.convert_alpha()
+                else:
+                    card = self.scaled_card.convert()
                 # center the image on the card
                 center_pos = img.get_rect()
                 center_pos.center = self.TargetSize.center # Wow, magic, see the pygame reference on Rect
@@ -767,10 +800,7 @@ class ScaleImages:
                            
         scale_x = int(imgSize[0] / scale_ratio)
         scale_y = int(imgSize[1] / scale_ratio)
-        if UT_DEBUG:
-            print "image size", imgSize
-            print "scale ratio", scale_ratio
-            print "scale x,y", scale_x, scale_y
+        
         scaled_img = pygame.transform.scale(img, (scale_x, scale_y))
         return scaled_img
 
@@ -889,85 +919,20 @@ class GfxCursor:
         """
         self.old_pos = event.pos
 
-class ProgressBar:
-    """Class which provides a surface with a progressbar.(400x100)
-    """
-    ## TODO: the progress bar is full after 95 steps, not 100.
-    ## The reason is that it's not possible to split pixels. :-)
-    def __init__(self, start=0, end=100, step=1, header='', ttf=None):
-        """@start is the start value, defaults to 0
-        @end is the end value, defaults to 100
-        @step is the step by which the bar is updated, defaults to 1
-        @header is the text to display above the bar, defaults to ''
-        @ttf is the path to a ttf, defaults to pygame standard font.
-        """
-        self.start = start
-        self.end = end
-        self.step = step
-        self.header = header
-        self.ttf = ttf
-        self.backcol = (0, 0, 0)
-        self.barfcol = (15, 206, 14)
-        self.barbcol = (20, 20, 20)
-        self.headercol = (200, 200, 200)
-        self.s = pygame.Surface((400, 100))
-        self.barpartsurf = pygame.Surface((4, 38))
-        self.barpartsurf.fill(self.barfcol)
-        self.barrect = pygame.Rect(10, 50, 380, 40)# relative to self.s
-        self.barsurf = pygame.Surface((self.barrect[2]-2, self.barrect[3]-2))
-        self._initbar()
-                
-    def _initbar(self):
-        self.x, self.progress = self.start, 0 # used in update
-        r = (0, 0) + self.s.get_size()
-        pygame.draw.rect(self.s, self.headercol, r, 1)
-        if self.header:
-            hs = char2surf(self.header, 24, self.headercol, self.ttf)
-            self.s.blit(hs, (4 + (400-hs.get_width()) / 2, 6))
-        pygame.draw.rect(self.s, self.headercol, self.barrect, 1)
-        self.barsurf.fill(self.barbcol)
-        self.s.blit(self.barsurf, (self.barrect[0] + 1, self.barrect[1] + 1))
-    
-    def update(self):
-        """Update the progressbar with step.
-        Returns the number of steps passed.
-        """
-        if self.progress >= self.end:
-            return self.end
-        for i in range(self.step):
-            self.barsurf.blit(self.barpartsurf, (self.x, 0))
-            self.x += self.barpartsurf.get_width()
-            self.progress += 1
-        self.s.blit(self.barsurf, (self.barrect[0] + 1, self.barrect[1] + 1))
-        return self.progress
-    
-    def reset_bar(self, header=''):
-        """This will set the bar to 0.
-        If you use a new header you MUST call clearbar first."""
-        if header:
-            self.header = header
-        self._initbar()
-        
-    def clearbar(self, screen, backgr):
-        """Remove the bar from the screen.
-        @ backgr is blitted over @screen.
-        This calls pygame.display.update"""
-        brect = self.s.get_rect().move(200, 200)
-        pygame.display.update(screen.blit(backgr, brect, brect))
-    
-    def get_bar(self):
-        return self.s
+
         
 def current_time():
     """Maincore uses this to get the current time to set the 'time_start'
     and 'time_end' values in the dbase table"""
-    return time.strftime("%y-%m-%d_%H:%M:%S", time.localtime())
+    t = time.strftime("%y-%m-%d_%H:%M:%S", time.localtime())
+    module_logger.debug("current_time returns %s" % t)
+    return t
 def calculate_time(start, end):
     """The 'end'time is substracted from 'start'time and the result time is
     returned in a format suitable to put into the dbase.
     Times must be in the format as returnt from current_time.
     """
-    logger = logging.getLogger("schoolsplay.utils.calculate_time")
+    logger = logging.getLogger("childsplay.utils.calculate_time")
     logger.debug("Called with start:%s, end:%s" % (start, end))
     start = start.replace('-', ':').replace('_', ':')
     end = end.replace('-', ':').replace('_', ':')
@@ -983,33 +948,40 @@ def calculate_time(start, end):
     dt2 = dt1 - dt0
     h, m = dt2.seconds / 60, dt2.seconds % 60
     return "%02d:%02d" % (h, m)
-    
-def shadefade(textstring, textsize, amount, displaysize, trgb=(255, 255, 255)):
+def sleep(ms):
+    """Pause the process for the given milliseconds and clears the event queue when
+    done."""
+    pygame.time.delay(ms)
+    pygame.event.clear()
+
+def shadefade(textstring, textsize, amount,bold=False, \
+              fcol=(200, 200, 200), shadecol=(255, 255, 255)):
     """Render @textstring onto a surface with a 3d dropshadow-like effect.
     Returns a SDL surface.
     Originally written by Brendan Becker, http://clickass.org/
     """
     if PLATFORM == 'win32':
         textsize -= 16
-    displaysurface = pygame.Surface(displaysize, SRCALPHA, 32)
-    font = pygame.font.Font(None, textsize)
+    
+    font = pangofont.PangoFont(None, textsize, bold=bold)
     # determine the size of the text surface so that we can center it later on
-    text = font.render(unicode(textstring), 1, (0, 0, 0))
-    width, height = text.get_size()
-    height = (displaysize[1] - height - amount) / 2
-    width = (displaysize[0] - width - amount) / 2
-    tempsurface = pygame.Surface((width + displaysize[0], height + displaysize[1]), SRCALPHA, 32)
+    basetext = font.render(unicode(textstring), 1, fcol, None)
+    width, height = basetext.get_size()
+    displaysurface = pygame.Surface((width, height), SRCALPHA, 32)
+    tempsurface = pygame.Surface((width + amount, height+amount), SRCALPHA, 32)
+    
     for i in range(amount):
         camt = amount-i
-        r = trgb[0] / camt
-        g = trgb[1] / camt
-        b = trgb[2] / camt
+        r = shadecol[0] / camt
+        g = shadecol[1] / camt
+        b = shadecol[2] / camt
         if r < 0:  r = 0
         if g < 0:  g = 0
         if b < 0:  b = 0
-        text = font.render(unicode(textstring), 1, (r, g, b))
+        text = font.render(unicode(textstring), 1, (r, g, b), None)
         tempsurface.blit(text, (camt, camt))
-    displaysurface.blit(tempsurface, (width, height))
+    tempsurface.blit(basetext, (0, 0))
+    displaysurface.blit(tempsurface, (0, 0))
     return displaysurface
 
 def _set_lock(timeout=10):
@@ -1024,7 +996,6 @@ def _set_lock(timeout=10):
         try:
             f = open(LOCKFILE, 'r')
             lines = f.readlines()
-            f.close()
             t = float(lines[0][:-1])
         except (IOError, TypeError):
             module_logger.exception("failed to read master lockfile")
@@ -1057,12 +1028,12 @@ def _remove_lock():
     if os.path.exists(LOCKFILE):
         try:
             os.remove(LOCKFILE)
-        except (OSError, IOError), info:
-            module_logger.exception("failed to remove master lockfile: %s" % info)
-            module_logger.error("Please remove lock file: %s manually" % LOCKFILE)
+        except IOError:
+            module_logger.exception("failed to remove master lockfile")
+            module_logger.error("Please remove lock file: % manually" % LOCKFILE)
     
-def Zscore(s_list, mean, sd):
-    """Returns the zscores in a list with the coresponding dates
+def score2percentile(s_list, mean, sd):
+    """Returns the percentile in a list with the coresponding dates
     calculated from the given parameters.
     @s_list is a list with tuples: [(date,score),(date,score)....]
     @mean is an float holding the mean from the dbase
@@ -1084,26 +1055,108 @@ def Zscore(s_list, mean, sd):
     RS is the student's raw score
     Mean is the mean (average) of the student raw scores
     SD is the Standard Deviation of the student raw scores.
+    
+    Then we find the percentile in a hash table and multiply it by 10 to get a 
+    nice value to display.
     """
-    module_logger.debug("Zscore called with:%s,%s,%s" % (s_list, mean, sd))
+    module_logger.debug("score2percentile called with:%s,%s,%s" % (s_list, mean, sd))
     # we unpack the score/date list, make the calculation and build
     # a new score/date list
     new_s_list = []
     for date, rs in s_list:
+        if not rs:
+           continue
         zs = (float(rs) - float(mean)) / float(sd)
-        module_logger.debug("rs: %s, mean:%s, sd:%s, zscore:%s" % (rs, mean, sd, zs))
-        new_s_list.append((date, zs))
+        if zs < 0:
+            prc = 1.0 - float(normal_distribution_table["%.2f" % abs(zs)])
+            prc = str(prc *10)
+        else:
+            prc = str(float(normal_distribution_table["%.2f" % zs]) * 10)[:4]
+        module_logger.debug("rs: %s, mean:%s, sd:%s, zscore:%s, percentile:%s" % (rs, mean, sd, zs, prc))
+        new_s_list.append((date, prc))
     return new_s_list
     
-def Percentile(z_list):
-    """Returns a list with all the percentile values and dates"""
-    pass
-    # See http://www.ehow.com/how_2310404_calculate-percentiles.html
-    # L/N * 100 == P
-    
-if __name__ == '__main__':
-    import SPLogging
-    SPLogging.set_level(CMD_Options.loglevel)
-    SPLogging.start()
-    
+class OrderedDict(dict, MutableMapping):
+    # Taken from Python 2.7, runs also on python  >= 2.4.
+    # Methods with direct access to underlying attributes
+    def __init__(self, *args, **kwds):
+        if len(args) > 1:
+            raise TypeError('expected at 1 argument, got %d', len(args))
+        if not hasattr(self, '_keys'):
+            self._keys = []
+        self.update(*args, **kwds)
+
+    def clear(self):
+        del self._keys[:]
+        dict.clear(self)
+
+    def __setitem__(self, key, value):
+        if key not in self:
+            self._keys.append(key)
+        dict.__setitem__(self, key, value)
+
+    def __delitem__(self, key):
+        dict.__delitem__(self, key)
+        self._keys.remove(key)
+
+    def __iter__(self):
+        return iter(self._keys)
+
+    def __reversed__(self):
+        return reversed(self._keys)
+
+    def popitem(self):
+        if not self:
+            raise KeyError
+        key = self._keys.pop()
+        value = dict.pop(self, key)
+        return key, value
+
+    def __reduce__(self):
+        items = [[k, self[k]] for k in self]
+        inst_dict = vars(self).copy()
+        inst_dict.pop('_keys', None)
+        return (self.__class__, (items,), inst_dict)
+
+    # Methods with indirect access via the above methods
+
+    setdefault = MutableMapping.setdefault
+    update = MutableMapping.update
+    pop = MutableMapping.pop
+    keys = MutableMapping.keys
+    values = MutableMapping.values
+    items = MutableMapping.items
+
+    def __repr__(self):
+        pairs = ', '.join(map('%r: %r'.__mod__, self.items()))
+        return '%s({%s})' % (self.__class__.__name__, pairs)
+
+    def copy(self):
+        return self.__class__(self)
+
+    @classmethod
+    def fromkeys(cls, iterable, value=None):
+        d = cls()
+        for key in iterable:
+            d[key] = value
+        return d
+
+#try:
+#    import db_conf as dbrc
+#    rckind = 'db_conf'
+#except ImportError:
+#    import db_dev as dbrc
+#    rckind = 'db_dev'
+#rc_hash = dbrc.rc
+#if rc_hash['default']['production']:
+#    kind = 'production'
+#else:
+#    kind = 'develop'
+#rc_hash['kind'] = kind
+#try:
+#    WEAREPRODUCTION = int(rc_hash['default']['production'])
+#except KeyError:
+#    WEAREPRODUCTION = '0'
+WEAREPRODUCTION = '0'
+
     
